'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.NGINX_PROXY = exports.MODULES = exports.UPLOAD_DOMAIN = exports.ASSETS_DOMAIN = exports.SERVER_DOMAIN = exports.CLIENT_DOMAIN = exports.DEVELOP_SERVER_PORT = exports.ENTRY_PATH = exports.DISTRICT_PATH = exports.EXEC_PATH = exports.ROOT_PATH = exports.RC_FILE = exports.ENTRY_FOLDER_NAME = exports.COVERAGE_FOLDER_NAME = exports.UNITEST_FOLDER_NAME = exports.DISTRICT_FOLDER_NAME = exports.VHOSTS_FOLDER_NAME = exports.DEVELOP_FOLDER_NAME = exports.TEMPORARY_FOLDER_NAME = exports.LOGGER_FOLDER_NAME = exports.RESOURCE_FOLDER_NAME = exports.PROJECT_NAME = undefined;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _package = require('../../package');

var _utils = require('../libs/utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var PROJECT_NAME = exports.PROJECT_NAME = _package.name;

var RESOURCE_FOLDER_NAME = exports.RESOURCE_FOLDER_NAME = 'src';
var LOGGER_FOLDER_NAME = exports.LOGGER_FOLDER_NAME = 'logs';
var TEMPORARY_FOLDER_NAME = exports.TEMPORARY_FOLDER_NAME = '.temporary';
var DEVELOP_FOLDER_NAME = exports.DEVELOP_FOLDER_NAME = '.dist';
var VHOSTS_FOLDER_NAME = exports.VHOSTS_FOLDER_NAME = 'vhosts';
var DISTRICT_FOLDER_NAME = exports.DISTRICT_FOLDER_NAME = 'dist';
var UNITEST_FOLDER_NAME = exports.UNITEST_FOLDER_NAME = 'unitest';
var COVERAGE_FOLDER_NAME = exports.COVERAGE_FOLDER_NAME = 'coverage';
var ENTRY_FOLDER_NAME = exports.ENTRY_FOLDER_NAME = 'app';

var RC_FILE = exports.RC_FILE = '.ngwprc';

var ROOT_PATH = exports.ROOT_PATH = process.cwd();
var EXEC_PATH = exports.EXEC_PATH = _path2.default.join(__dirname, '../../');
var DISTRICT_PATH = exports.DISTRICT_PATH = _path2.default.join(ROOT_PATH, process.env.DEVELOP ? DEVELOP_FOLDER_NAME : DISTRICT_FOLDER_NAME);
var ENTRY_PATH = exports.ENTRY_PATH = _path2.default.join(ROOT_PATH, RESOURCE_FOLDER_NAME, ENTRY_FOLDER_NAME);

var DEVELOP_SERVER_PORT = exports.DEVELOP_SERVER_PORT = 50000;

var CLIENT_DOMAIN = exports.CLIENT_DOMAIN = 'www.' + _path2.default.basename(ROOT_PATH).toLowerCase() + '.com';
var SERVER_DOMAIN = exports.SERVER_DOMAIN = '';
var ASSETS_DOMAIN = exports.ASSETS_DOMAIN = '';
var UPLOAD_DOMAIN = exports.UPLOAD_DOMAIN = '';

var MODULES = exports.MODULES = [];
var NGINX_PROXY = exports.NGINX_PROXY = [];

/**
 * Setting domains and develop server port
 */
var rc = _path2.default.join(ROOT_PATH, '.ngwprc');
if (_fsExtra2.default.existsSync(rc)) {
  var source = _fsExtra2.default.readJsonSync(rc);

  exports.DEVELOP_SERVER_PORT = DEVELOP_SERVER_PORT = source.port || DEVELOP_SERVER_PORT;
  exports.CLIENT_DOMAIN = CLIENT_DOMAIN = source.clientDomain || CLIENT_DOMAIN;
  exports.SERVER_DOMAIN = SERVER_DOMAIN = source.serverDomain || SERVER_DOMAIN;
  exports.ASSETS_DOMAIN = ASSETS_DOMAIN = source.assetsDomain || ASSETS_DOMAIN;
  exports.UPLOAD_DOMAIN = UPLOAD_DOMAIN = source.uploadDomain || UPLOAD_DOMAIN;
  exports.NGINX_PROXY = NGINX_PROXY = source.nginxProxy || NGINX_PROXY;
}

/**
 * Generate modules from app folder.
 * folder: /app_resource/{module_name}/
 */
if (_lodash2.default.isEmpty(MODULES) && _fsExtra2.default.existsSync(ENTRY_PATH) && _fsExtra2.default.lstatSync(ENTRY_PATH).isDirectory()) {
  var modules = _fsExtra2.default.readdirSync(ENTRY_PATH);

  modules = _lodash2.default.filter(modules, findModule);
  modules = _lodash2.default.map(modules, _utils.convertName);
  modules = _lodash2.default.map(modules, 'underscore');
  exports.MODULES = MODULES = MODULES.concat(modules);
}

/**
 * generate nginx proxy config
 */
if (_lodash2.default.isEmpty(NGINX_PROXY)) {
  if (!_lodash2.default.isEmpty(MODULES)) {
    var proxy = {
      type: 'proxy',
      proxy: '127.0.0.1',
      proxyPort: DEVELOP_SERVER_PORT,
      entries: MODULES,
      domain: [CLIENT_DOMAIN]
    };

    NGINX_PROXY.push(proxy);
  }
} else {
  exports.NGINX_PROXY = NGINX_PROXY = _lodash2.default.map(NGINX_PROXY, function (proxy) {
    return _lodash2.default.defaultsDeep(proxy, {
      type: 'proxy',
      proxy: '127.0.0.1',
      proxyPort: DEVELOP_SERVER_PORT
    });
  });
}

if (isValidDomain(ASSETS_DOMAIN)) {
  var domains = [ASSETS_DOMAIN];

  if (UPLOAD_DOMAIN !== ASSETS_DOMAIN && isValidDomain(UPLOAD_DOMAIN)) {
    domains.push(UPLOAD_DOMAIN);
  }

  var _proxy = {
    type: 'cdn',
    domain: domains
  };

  NGINX_PROXY.push(_proxy);
}

/**
 * filter valid module
 * @param  {String} name module name
 * @return {Boolean}
 */
function findModule(name) {
  return (/^[\w\d\_\-]+$/.test(name)
  );
}

/**
 * valid domain
 * - check valid chars
 * - check overall length
 * - check length of each label
 * @param  {String}  domain Domain name
 * @return {Boolean}
 */
function isValidDomain(domain) {
  return (/^([a-z\d](-*[a-z\d])*)(\.([a-z\d](-*[a-z\d])*))*$/i.test(domain) && /^.{1,253}$/.test(domain) && /^[^\.]{1,63}(\.[^\.]{1,63})*$/.test(domain)
  );
}