'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

exports.generateEnteries = generateEnteries;
exports.generateFavicons = generateFavicons;
exports.generateSprites = generateSprites;
exports.generateSVGSprites = generateSVGSprites;
exports.widthDone = widthDone;
exports.injectScript = injectScript;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _cleanWebpackPlugin = require('clean-webpack-plugin');

var _cleanWebpackPlugin2 = _interopRequireDefault(_cleanWebpackPlugin);

var _spritesheetTemplates = require('spritesheet-templates');

var _spritesheetTemplates2 = _interopRequireDefault(_spritesheetTemplates);

var _webpackSpritesmith = require('webpack-spritesmith');

var _webpackSpritesmith2 = _interopRequireDefault(_webpackSpritesmith);

var _webpackSvgstorePlugin = require('webpack-svgstore-plugin');

var _webpackSvgstorePlugin2 = _interopRequireDefault(_webpackSvgstorePlugin);

var _faviconsWebpackPlugin = require('favicons-webpack-plugin');

var _faviconsWebpackPlugin2 = _interopRequireDefault(_faviconsWebpackPlugin);

var _htmlWebpackPlugin = require('html-webpack-plugin');

var _htmlWebpackPlugin2 = _interopRequireDefault(_htmlWebpackPlugin);

var _autoprefixer = require('autoprefixer');

var _autoprefixer2 = _interopRequireDefault(_autoprefixer);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _copyWebpackPlugin = require('copy-webpack-plugin');

var _copyWebpackPlugin2 = _interopRequireDefault(_copyWebpackPlugin);

var _variables = require('./variables');

var VARS = _interopRequireWildcard(_variables);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Entries definitions
 */
var entries = {};

/**
 * reolve path definitions
 */
var resolveModules = [_path2.default.join(VARS.EXEC_PATH, 'node_modules'), _path2.default.join(VARS.ROOT_PATH, 'node_modules'), _path2.default.join(VARS.ROOT_PATH, VARS.TEMPORARY_FOLDER_NAME), _path2.default.join(VARS.ROOT_PATH, VARS.RESOURCE_FOLDER_NAME)];

/**
 * Plugins definitions
 */
var plugins = [
/**
 * set environment variables
 * you can find out global variables in javascript
 *
 * if you want to set __VALUE_ equal 'string' (__VALUE__ = 'string')
 * yout muse define { __VALUE__ : '\"string\"' }
 */
new _webpack2.default.DefinePlugin({
  __DEVELOP__: !!process.env.DEVELOP,
  __PRODUCT__: !!process.env.PRODUCT,
  __UNITEST__: !!process.env.UNITEST,

  __CLIENT_DOMAIN__: JSON.stringify(VARS.CLIENT_DOMAIN),
  __ASSETS_DOMAIN__: JSON.stringify(VARS.ASSETS_DOMAIN),
  __UPLOAD_DOMAIN__: JSON.stringify(VARS.UPLOAD_DOMAIN),
  __SERVER_DOMAIN__: JSON.stringify(VARS.SERVER_DOMAIN)
}),

/**
 * Extract common modules
 * to reduce code duplication
 */
new _webpack2.default.optimize.CommonsChunkPlugin({
  name: 'vendor',
  chunks: [],
  minChunks: function minChunks(module) {
    var folder = _path2.default.resolve(VARS.ROOT_PATH, VARS.RESOURCE_FOLDER_NAME);
    return module.resource && -1 === module.resource.indexOf(folder);
  }
}),

/**
 * Extract style file
 * Inline styles can be externally optimized for loading
 */
new _extractTextWebpackPlugin2.default({
  filename: 'styles/[name].[contenthash].css',
  allChunks: true
}),

/**
 * Copy files
 */
new _copyWebpackPlugin2.default([{
  from: _path2.default.join(VARS.ROOT_PATH, VARS.RESOURCE_FOLDER_NAME, 'assets/panels/**'),
  to: _path2.default.join(VARS.DISTRICT_PATH, 'assets/panels/'),
  flatten: true
}]),

/**
 * Clean generate folders
 * run it first to reset the project.
 */
new _cleanWebpackPlugin2.default([VARS.TEMPORARY_FOLDER_NAME, VARS.DEVELOP_FOLDER_NAME, VARS.DISTRICT_FOLDER_NAME, VARS.COVERAGE_FOLDER_NAME], {
  root: VARS.ROOT_PATH,
  verbose: true,
  dry: false
})];

// let Injector = injectScript(plugins);
var CallAfter = widthDone(plugins);

/**
 * Generate some compile tasks
 */
generateEnteries(plugins, entries);
generateFavicons(plugins);
var spriteGenerated = generateSprites(plugins);
// generateSVGSprites(plugins);

/**
 * some browser will request '/favicon.ico' file
 * wechat browser in ios reset title will also request '/favicon.ico'
 * (use iframe to request favicon.ico file)
 */
var faviconFile = _path2.default.join(VARS.DISTRICT_PATH, 'favicon.ico');
_fsExtra2.default.ensureFileSync(faviconFile);

/**
 * loader and rules definitions
 */
var rules = [{
  test: /\.html$/,
  use: [{
    loader: 'html-loader',
    options: {
      attrs: ['img:src', 'img:ng-src']
    }
  }]
}, {
  test: /\.jade$/,
  use: [{
    loader: 'pug-loader'
  }]
}, {
  test: /\.css$/,
  enforce: 'pre',
  exclude: [/node_modules/],
  loader: 'stylelint-loader',
  options: {
    configFile: backup(_path2.default.join(VARS.ROOT_PATH, '.stylelintrc'), _path2.default.join(VARS.EXEC_PATH, '.stylelintrc'))
  }
},
/**
 * As Jade/Pug will use require() to load public style
 * like bootstrap.css, so that we must provider a loader
 * to load the file.
 * At the same time, `ExtractTextPlugin` plugin do not
 * match .css file, because it will throw an error to
 * tell you no loader can load this file.
 *
 * Error:
 *   Module build failed:
 *   Error: "extract-text-webpack-plugin" loader is used
 *   without the corresponding plugin, refer to
 *   https://github.com/webpack/extract-text-webpack-plugin
 *   for the usage example
 */
{
  test: /\.css$/,
  use: {
    loader: 'url-loader',
    options: {
      limit: 10000,
      name: 'styles/[name].[hash].css'
    }
  }
},
/**
 * docs:
 * - https://github.com/webpack-contrib/extract-text-webpack-plugin/issues/263
 */
{
  test: /\.(sass|scss)$/,
  use: _extractTextWebpackPlugin2.default.extract({
    fallback: 'style-loader',
    use: [{
      loader: 'css-loader',
      options: {
        sourceMap: true
      }
    }, {
      loader: 'sass-loader',
      options: {
        includePaths: resolveModules,
        data: [spriteGenerated ? '@import "sprites";' : ''].join('\n')
      }
    }, {
      loader: 'postcss-loader',
      options: {
        plugins: [(0, _autoprefixer2.default)({
          browsers: ['last 10 version', 'ie >= 9']
        })]
      }
    }]
  })
}, {
  test: /\.js$/,
  enforce: 'pre',
  exclude: [/node_modules/],
  loader: 'eslint-loader',
  options: {
    configFile: backup(_path2.default.join(VARS.ROOT_PATH, '.eslintrc'), _path2.default.join(VARS.EXEC_PATH, '.eslintrc'))
  }
}, {
  test: /\.js$/,
  use: [{
    loader: 'ng-annotate-loader'
  },
  /**
   * babel@6.0.0 break the .babelrc file
   * so configure presets below
   * docs:
   * - https://github.com/babel/babel-loader/issues/166
   */
  {
    loader: 'babel-loader',
    options: {
      presets: [require.resolve('babel-preset-es2015'), require.resolve('babel-preset-stage-0')]
    }
  }]
},
/**
 * 少于 10K 图片用 base64
 * url-loader 依赖 file-loader
 */
{
  test: /\.(jpe?g|png|gif)$/i,
  use: [{
    loader: 'url-loader',
    options: {
      limit: 10000,
      name: 'panels/[name].[hash].[ext]'
    }
  }]
}];

/**
 * Webpack Setting
 */
exports.default = {
  entry: entries,
  output: {
    path: VARS.DISTRICT_PATH,
    publicPath: '/',
    filename: '[name].js'
  },
  module: {
    rules: rules
  },
  resolve: {
    modules: resolveModules
  },
  resolveLoader: {
    modules: resolveModules
  },
  plugins: plugins
};

/**
 * Auto generate entries
 * Generate entries dependent on folder (src/entry/{folder})
 * And entry js file must be named 'index.js'
 */

function generateEnteries(plugins, entries) {
  if (!_lodash2.default.isArray(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  if (!_lodash2.default.isObject(entries)) {
    throw new Error('Parameter entries must be a object.');
  }

  if (_fsExtra2.default.existsSync(VARS.ENTRY_PATH) && _fsExtra2.default.lstatSync(VARS.ENTRY_PATH).isDirectory()) {
    var modules = _fsExtra2.default.readdirSync(VARS.ENTRY_PATH);

    if (0 === modules.length) {
      return false;
    }

    modules.forEach(function (name) {
      var dir = _path2.default.join(VARS.ENTRY_PATH, name);

      if (_fsExtra2.default.statSync(dir).isDirectory()) {
        var bootstrapFile = _path2.default.join(dir, 'index.js');

        if (_fsExtra2.default.existsSync(bootstrapFile)) {
          entries[name] = bootstrapFile;

          /**
           * reanme entry html
           */
          var options = {
            filename: _path2.default.join(VARS.DISTRICT_PATH, name + '.html')
          };

          /**
           * use template when then template file exists
           */
          var tmplFile = _path2.default.join(VARS.ENTRY_PATH, name + '/index.jade');
          if (_fsExtra2.default.existsSync(tmplFile)) {
            Object.assign(options, {
              template: tmplFile
            });
          }

          /**
           * clean other static resources
           */
          Object.assign(options, {
            excludeChunks: _lodash2.default.without(modules, name)
          });

          var plugin = new _htmlWebpackPlugin2.default(options);
          plugins.push(plugin);
        }
      }
    });

    return true;
  }

  return false;
}

/**
 * auto split logo task
 * if logo file not exists, this task will not be executed.
 */
function generateFavicons(plugins) {
  if (!_lodash2.default.isArray(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  var LOGO_FILE = _path2.default.join(VARS.ROOT_PATH, VARS.RESOURCE_FOLDER_NAME, 'assets/panels/logo.png');

  if (_fsExtra2.default.existsSync(LOGO_FILE)) {
    var statsFile = 'favicon/iconstats.json';

    var plugin = new _faviconsWebpackPlugin2.default({
      logo: LOGO_FILE,
      prefix: 'favicon/[hash]/',
      emitStats: true,
      statsFilename: statsFile,
      persistentCache: true,
      inject: true,
      background: '#fff',
      icons: {
        android: true,
        appleIcon: true,
        appleStartup: false,
        coast: false,
        favicons: true,
        firefox: true,
        opengraph: false,
        twitter: false,
        yandex: false,
        windows: false
      }
    });

    plugins.push(plugin);

    /**
     * after compile, it will generate 'favicon.ico' file
     * and copy it to the root path.
     */
    CallAfter.add(function () {
      var sourceFile = _path2.default.join(VARS.DISTRICT_PATH, statsFile);
      if (!_fsExtra2.default.existsSync(sourceFile)) {
        return;
      }

      var stats = _fsExtra2.default.readJsonSync(sourceFile);
      var favFile = _path2.default.join(VARS.DISTRICT_PATH, stats.outputFilePrefix, 'favicon.ico');

      _fsExtra2.default.copySync(favFile, faviconFile);
    });

    return true;
  }

  return false;
}

/**
 * Auto concat sprite image
 * if the sprite folder (src/assets/sprites/images/) not exists,
 * this task will not be excuted.
 */
function generateSprites(plugins) {
  var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  if (!_lodash2.default.isArray(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  var SPRITE_DIR = options.resources || _path2.default.join(VARS.ROOT_PATH, VARS.RESOURCE_FOLDER_NAME, 'assets/sprites/images');
  var SPRITE_TEMPLATE_FILE = _path2.default.join(SPRITE_DIR, 'sprite.scss.template.handlebars');

  if (_fsExtra2.default.existsSync(SPRITE_DIR) && _fsExtra2.default.lstatSync(SPRITE_DIR).isDirectory() && _fsExtra2.default.existsSync(SPRITE_TEMPLATE_FILE)) {
    var source = _fsExtra2.default.readFileSync(SPRITE_TEMPLATE_FILE, 'utf8');
    _spritesheetTemplates2.default.addHandlebarsTemplate('spriteScssTemplate', source);

    var plugin = new _webpackSpritesmith2.default({
      src: {
        cwd: SPRITE_DIR,
        glob: '**/*.{png,gif,jpg}'
      },
      target: {
        image: _path2.default.join(VARS.TEMPORARY_FOLDER_NAME, 'sprites.png'),
        css: [[_path2.default.join(VARS.TEMPORARY_FOLDER_NAME, 'sprites.scss'), {
          format: 'spriteScssTemplate'
        }]]
      },
      apiOptions: {
        cssImageRef: '~sprites.png'
      },
      spritesmithOptions: {
        functions: true,
        padding: 10
      }
    });

    plugins.push(plugin);

    return true;
  }

  return false;
}

/**
 * Auto concat svg sprite image
 * if the sprite folder (src/assets/sprites/svg/) not exists,
 * this task will not be excuted.
 */
function generateSVGSprites(plugins) {
  if (!_lodash2.default.isArray(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  var SPRITE_DIR = _path2.default.join(VARS.RESOURCE_FOLDER_NAME, 'assets/sprites/svg');
  var SPRITE_CONFIG_FILE = _path2.default.join(SPRITE_DIR, 'svgstore.config.js');

  if (_fsExtra2.default.existsSync(SPRITE_DIR) && _fsExtra2.default.lstatSync(SPRITE_DIR).isDirectory() && _fsExtra2.default.existsSync(SPRITE_CONFIG_FILE)) {
    Object.assign(entries, {
      svgstore: _path2.default.join(VARS.ROOT_PATH, SPRITE_CONFIG_FILE)
    });

    var plugin = new _webpackSvgstorePlugin2.default({
      prefix: 'sp-svg-',
      svgoOptions: {
        plugins: [{ removeComments: true }, { removeMetadata: true }, { removeTitle: true }, { removeDesc: true }, { removeUselessDefs: true }, { removeXMLNS: true }, { minifyStyles: true }, { cleanupIDs: true }, { removeEmptyText: true }, { convertColors: true }, { convertPathData: true }, { convertTransform: true }, { removeUnknownsAndDefaults: true }, { removeUnusedNS: true },
        /**
         * svg in webkit old browser, it not support use (reference)
         * it must use '<use xlink:href="url#id"></use>'
         * and because svgo(https://github.com/svg/svgo) do not set
         * 'xmlns:xlink="http://www.w3.org/1999/xlink"', so it make
         * origin svg content with use tag lack 'namespace' 'prefix',
         * and it make svg display success.
         *
         * SVG 在 webkit 低版本浏览器中不支持内联 use
         * 必须使用 <use xlink:href="url#id"></use>
         * 又因为 svgo 并没有设置 xmlns:xlink="http://www.w3.org/1999/xlink"
         * 因此会使原本 svg 内含有 use 缺少 namespace prefix 的问题, 导致没法兼容加载,
         * 导致外部不能成功导入
         *
         * Error Code:
         * This page contains the following errors:
         * error on line 1 at column 15734: Namespace prefix xlink for href on use is not defined
         * Below is a rendering of the page up to the first error.
         *
         * Browser: Chrome 48.0.2564.23:
         * Mozilla/5.0 (Linux; Android 6.0; Nexus 5 Build/MRA58N)
         * AppleWebKit/537.36 (KHTML, like Gecko)
         * Chrome/48.0.2564.23
         * Mobile Safari/537.36
         * wechatdevtools/0.7.0
         * MicroMessenger/6.3.22
         * webview/0
         *
         * Docs : https://github.com/svg/svgo/blob/master/docs/how-it-works/en.md#3-plugins
         * API  : https://github.com/svg/svgo/blob/master/docs/how-it-works/en.md#32-api
         */
        {
          downwardCompatible: {
            type: 'perItem',
            active: true,
            description: 'Backward compatibility, <use> add attribute xmlns:xlink="http://www.w3.org/1999/xlink"',
            params: {},
            fn: function fn(item) {
              if (item.isElem('use') && !hasAttr(item, 'xmlns:xlink')) {
                setAttr(item, 'xmlns:xlink', 'http://www.w3.org/1999/xlink');
              }
            }
          }
        }]
      }
    });

    plugins.push(plugin);
    return true;
  }

  return false;

  function hasAttr(item, name) {
    var _name$split = name.split(':'),
        _name$split2 = _slicedToArray(_name$split, 2),
        prefix = _name$split2[0],
        local = _name$split2[1];

    return -1 !== _lodash2.default.indexOf(item.attrs, { name: name, prefix: prefix, local: local });
  }

  function setAttr(item, name, value) {
    var _name$split3 = name.split(':'),
        _name$split4 = _slicedToArray(_name$split3, 2),
        prefix = _name$split4[0],
        local = _name$split4[1];

    item.attrs[name] = { value: value, name: name, prefix: prefix, local: local };
  }
}

/**
 * Callback after webpack excutes
 */
function widthDone(plugins) {
  if (!_lodash2.default.isArray(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  var instance = {
    _callbacks: [],
    add: function add(callback) {
      _lodash2.default.isFunction(callback) && this._callbacks.push(callback);
    },
    done: function done() {
      _lodash2.default.forEach(this._callbacks, function (callback) {
        callback();
      });
    }
  };

  plugins.push({
    apply: function apply(compiler) {
      compiler.plugin('done', instance.done.bind(instance));
    }
  });

  return instance;
}

/**
 * Inject script to entry html file
 */
function injectScript(plugins) {
  if (!_lodash2.default.isArray(plugins)) {
    throw new Error('Parameter plugins must be a array.');
  }

  var instance = {
    _injector: [],
    _callbacks: [],
    inject: function inject(source) {
      if (!_lodash2.default.isString(source)) {
        return false;
      }

      var hash = mkhash(source);
      if (-1 !== _lodash2.default.indexOf(this._injector, { hash: hash })) {
        return false;
      }

      var script = '!(function () {\n        var id = \'webpack-' + VARS.PROJECT_NAME + '-v' + hash + '\';\n        if (\'undefined\' === typeof window || document.getElementById(id)) {\n          return;\n        }\n\n        var node       = document.createElement(\'script\');\n        node.id        = id;\n        node.innerHTML = \'' + source + '\';\n\n        document.head.appendChild(node);\n      })();';

      this._injector.push({ hash: hash, script: script });
    },
    after: function after(callback) {
      _lodash2.default.isFunction(callback) && this._callbacks.push(callback);
    }
  };

  plugins.push({
    autoloadScript: function autoloadScript() {
      var scripts = [];

      _lodash2.default.forEach(instance._injector, function (injector) {
        scripts.push(injector.script);
      });

      return scripts.join('\n');
    },
    scriptTag: function scriptTag(source) {
      var injector = this.autoloadScript();
      return injector + source;
    },
    applyCompilation: function applyCompilation(compilation) {
      compilation.mainTemplate.plugin('startup', this.scriptTag.bind(this));
    },
    applyDone: function applyDone() {
      _lodash2.default.forEach(instance._callbacks, function (injector) {
        injector();
      });

      instance._callbacks.splice(0);
    },
    apply: function apply(compiler) {
      /**
       * sometimes it will trigger twice or more,
       * the core-code just exec once, see below function autoloadScript.
       */
      compiler.plugin('compilation', this.applyCompilation.bind(this));
      compiler.plugin('done', this.applyDone.bind(this));
    }
  });

  return instance;
}

/**
 * make hash code
 * See: http://stackoverflow.com/questions/7616461/generate-a-hash-from-string-in-javascript-jquery?answertab=active#tab-top
 */
function mkhash(string) {
  string = JSON.stringify(string);

  var hash = 0;
  if (0 === string.length) {
    return hash;
  }

  for (var i = 0, l = string.length; i < l; i++) {
    var chr = string.charCodeAt(i);
    hash = (hash << 5) - hash + chr;

    /**
     * Convert to 32bit integer
     */
    hash |= 0;
  }

  return hash;
}

function backup(file, bkfile) {
  if (!_fsExtra2.default.existsSync(file)) {
    return bkfile;
  }

  return file;
}