'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackMerge = require('webpack-merge');

var _webpackMerge2 = _interopRequireDefault(_webpackMerge);

var _cleanWebpackPlugin = require('clean-webpack-plugin');

var _cleanWebpackPlugin2 = _interopRequireDefault(_cleanWebpackPlugin);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _variables = require('./variables');

var VARS = _interopRequireWildcard(_variables);

var _webpackCommonConfig = require('./webpack.common.config.babel');

var _webpackCommonConfig2 = _interopRequireDefault(_webpackCommonConfig);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Entries definitions
 */
var entries = {
  'babel-polyfill': 'babel-polyfill'
};

/**
 * Plugins definitions
 */
var plugins = [
/**
 * set environment variables
 * you can find out global variables in javascript
 *
 * if you want to set __VALUE_ equal 'string' (__VALUE__ = 'string')
 * yout muse define { __VALUE__ : '\"string\"' }
 */
new _webpack2.default.DefinePlugin({
  __DEVELOP__: !!process.env.DEVELOP,
  __PRODUCT__: !!process.env.PRODUCT,
  __UNITEST__: !!process.env.UNITEST,

  __CLIENT_DOMAIN__: JSON.stringify(VARS.CLIENT_DOMAIN),
  __ASSETS_DOMAIN__: JSON.stringify(VARS.ASSETS_DOMAIN),
  __UPLOAD_DOMAIN__: JSON.stringify(VARS.UPLOAD_DOMAIN),
  __SERVER_DOMAIN__: JSON.stringify(VARS.SERVER_DOMAIN)
}),

/**
 * Extract style file
 * Inline styles can be externally optimized for loading
 */
new _extractTextWebpackPlugin2.default({
  filename: 'styles/[name].[contenthash].css',
  allChunks: true
}),

/**
 * Clean generate folders
 * run it first to reset the project.
 */
new _cleanWebpackPlugin2.default([VARS.DEVELOP_FOLDER_NAME, VARS.DISTRICT_FOLDER_NAME, VARS.COVERAGE_FOLDER_NAME], {
  root: VARS.ROOT_PATH,
  verbose: true,
  dry: false
})];

/**
 * Generate some compile tasks
 */
(0, _webpackCommonConfig.generateSprites)(plugins);

exports.default = (0, _webpackMerge2.default)({
  devtool: 'inline-source-map',
  entry: entries,
  output: _webpackCommonConfig2.default.output,
  module: _webpackCommonConfig2.default.module,
  resolve: _webpackCommonConfig2.default.resolve,
  resolveLoader: _webpackCommonConfig2.default.resolveLoader,
  plugins: plugins
},
/**
 * Sinon setting
 *
 * UMD will make compiled occur some error
 * Error:
 * modules[moduleId].call is not a function
 * Issue: https://github.com/webpack/webpack/issues/304
 *
 * @todo 后面版本修复后删除
 */
{
  module: {
    noParse: [/node_modules\/sinon\//]
  },
  resolve: {
    alias: {
      sinon: 'sinon/pkg/sinon.js'
    }
  }
},
/**
 * unitest setting
 */
{
  // module: {
  //   rules: [
  //     {
  //       test    : /\.js$/,
  //       enforce : 'pre',
  //       loader  : 'istanbul-instrumenter-loader',
  //       include : webpackConfig.resolve.modules,
  //       exclude : [/node_modules/, new RegExp(VARS.TEMPORARY_FOLDER_NAME)],
  //     }
  //   ],
  // },
});