'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.initialize = initialize;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _utils = require('./utils');

var _option_merger = require('./option_merger');

var _option_merger2 = _interopRequireDefault(_option_merger);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * initialize project
 * @param {String}   name                 project name
 * @param {Object}   options              initialization setting
 * @param {String}   options.version      project version (default 1.0.0)
 * @param {String}   options.description  project description (default project name)
 * @param {String}   dist                 output folder (default pwd)
 * @param {Function} callback             callback after exec
 */
function initialize(name, options, callback) {
  if (!_lodash2.default.isString(name)) {
    throw new Error('Name is not provided');
  }

  if (!_lodash2.default.isFunction(callback)) {
    throw new Error('Callback is not provided');
  }

  var tplFolder = _path2.default.join(_option_merger2.default.EXEC_PATH, './templates/project');
  var tarFolder = options.dist || _option_merger2.default.ROOT_PATH;

  _fsExtra2.default.ensureDirSync(tarFolder);

  var keys = ['LOGGER_FOLDER_NAME', 'TEMPORARY_FOLDER_NAME', 'DEVELOP_FOLDER_NAME', 'DISTRICT_FOLDER_NAME', 'COVERAGE_FOLDER_NAME', 'VHOSTS_FOLDER_NAME'];

  var gitIgnore = _lodash2.default.pick(_option_merger2.default, keys);
  var gitIngoreFolders = _lodash2.default.values(gitIgnore);

  /**
   * only set root directory
   */
  gitIngoreFolders = _lodash2.default.map(gitIngoreFolders, function (folder) {
    return '/' + folder;
  });

  (0, _utils.copyAndRender)(tplFolder, tarFolder, {
    name: name,
    version: options.version || '0.0.1',
    description: options.description || 'Project ' + name,
    gitIngoreFolders: gitIngoreFolders
  }, callback);
}