'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.mkVhost = mkVhost;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _handlebars = require('handlebars');

var _handlebars2 = _interopRequireDefault(_handlebars);

var _utils = require('../libs/utils');

var utils = _interopRequireWildcard(_utils);

var _variables = require('../conf/variables');

var VARS = _interopRequireWildcard(_variables);

var _option_merger = require('./option_merger');

var _option_merger2 = _interopRequireDefault(_option_merger);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Register Handlebars helpers
 * @docs: http://handlebarsjs.com/block_helpers.html
 */

/**
 * Compare number and type-equals with variables
 */
_handlebars2.default.registerHelper('compare', compare);

/**
 * Separate Array to some string.
 * [value1, value2, value3] => 'value1 value2 value3';
 */
_handlebars2.default.registerHelper('separate', separate);

/**
 * build nginx vhosts
 * @param  {Array}    modules  module setting
 * @param  {Object}   options  setting
 * @param  {Function} callback result callback function
 */
function mkVhost(modules, options, callback) {
  if (!_lodash2.default.isFunction(callback)) {
    throw new Error('Callback is not provided.');
  }

  var basePath = options.basePath || process.cwd();

  options = _lodash2.default.defaultsDeep(options, {
    trace: false,

    distFile: _path2.default.join(basePath, 'vhosts/nginx.conf'),
    template: _path2.default.join(_option_merger2.default.EXEC_PATH, './templates/vhosts/nginx.conf.hbs'),
    rootPath: _path2.default.join(basePath, VARS.DISTRICT_FOLDER_NAME),
    logsPath: _path2.default.join(basePath, VARS.LOGGER_FOLDER_NAME),
    useHttps: _lodash2.default.isBoolean(options.useHttps) ? options.useHttps : false
  });

  if (!_fsExtra2.default.existsSync(options.template)) {
    callback(new Error('Template \'' + options.template + '\' is not exists.'));
    return;
  }

  var template = _fsExtra2.default.readFileSync(options.template, 'utf-8');
  var compile = _handlebars2.default.compile(template);

  modules = _lodash2.default.cloneDeep(modules);

  var _iteratorNormalCompletion = true;
  var _didIteratorError = false;
  var _iteratorError = undefined;

  try {
    for (var _iterator = modules[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
      var module = _step.value;

      if (!(_lodash2.default.isString(module.domain) || _lodash2.default.isArray(module.domain)) && _lodash2.default.isEmpty(module.domain)) {
        callback(new Error('Domain is not provided.'));
        return;
      }

      if ('proxy' === module.type) {
        if (!(_lodash2.default.isArray(module.entries) && 0 < module.entries.length)) {
          callback(new Error('Entries is not provided.'));
          return;
        }

        if (!(_lodash2.default.isString(module.proxy) && /^(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])\.(\d{1,2}|1\d\d|2[0-4]\d|25[0-5])$/.exec(module.proxy))) {
          callback(new Error('Proxy is not provided or invalid. (Proxy is a ip address, eg: 127.0.0.1)'));
          return;
        }

        if (!_lodash2.default.isNumber(module.proxyPort)) {
          callback(new Error('ProxyPort is not provided or invalid. (ProxyPort must be a port number)'));
          return;
        }
      }

      if (true === options.useHttps && true === module.useHttps) {
        if (!module.certFile) {
          callback(new Error('CertFile is not provided when use https'));
          return;
        }

        var certFile = utils.resolvePath(module.certFile, options.certPath);
        if (!_fsExtra2.default.existsSync(certFile)) {
          callback(new Error('CertFile ' + certFile + ' is not found'));
          return;
        }

        if (!module.certKey) {
          callback(new Error('CertKey is not provided when use https'));
          return;
        }

        var certKey = utils.resolvePath(module.certKey, options.certPath);
        if (!_fsExtra2.default.existsSync(certKey)) {
          callback(new Error('CertKey ' + certKey + ' is not found'));
          return;
        }

        module.certFile = certFile;
        module.certKey = certKey;
      }

      if (_lodash2.default.isArray(module.entries)) {
        Object.assign(module, {
          division: module.entries.join('|'),
          logger: _lodash2.default.isArray(module.domain) ? module.domain[0] : module.domain
        });
      }
    }
  } catch (err) {
    _didIteratorError = true;
    _iteratorError = err;
  } finally {
    try {
      if (!_iteratorNormalCompletion && _iterator.return) {
        _iterator.return();
      }
    } finally {
      if (_didIteratorError) {
        throw _iteratorError;
      }
    }
  }

  _fsExtra2.default.ensureDirSync(options.logsPath);

  var source = compile({
    rootPath: options.rootPath,
    logsPath: options.logsPath,
    modules: modules
  });

  _fsExtra2.default.ensureDir(options.distFile.replace(_path2.default.basename(options.distFile), ''));
  _fsExtra2.default.writeFile(options.distFile, source, function (error) {
    if (error) {
      callback(error);
      return;
    }

    callback(null, { file: options.distFile, modules: modules });
  });
}

/**
 * compare values
 * @param  {String} lvalue   first value
 * @param  {String} operator operator symbol
 * @param  {String} rvalue   second value
 * @param  {Object} options
 */
function compare(lvalue, operator, rvalue, options) {
  var operators = void 0;
  var result = void 0;

  if (3 > arguments.length) {
    throw new Error('Handlerbars Helper "compare" needs 2 parameters');
  }

  if (undefined === options) {
    options = rvalue;
    rvalue = operator;
    operator = '===';
  }

  operators = {
    '==': function _(l, r) {
      /* eslint eqeqeq:off */
      return l == r;
    },
    '===': function _(l, r) {
      return l === r;
    },
    '!=': function _(l, r) {
      /* eslint eqeqeq:off */
      return l != r;
    },
    '!==': function _(l, r) {
      return l !== r;
    },
    '<': function _(l, r) {
      return l < r;
    },
    '>': function _(l, r) {
      return l > r;
    },
    '<=': function _(l, r) {
      return l <= r;
    },
    '>=': function _(l, r) {
      return l >= r;
    },
    'typeof': function _typeof(l, r) {
      /* eslint eqeqeq:off */
      return (typeof l === 'undefined' ? 'undefined' : _typeof2(l)) == r;
    }
  };

  if (!operators[operator]) {
    throw new Error('Handlerbars Helper \'compare\' doesn\'t know the operator ' + operator);
  }

  result = operators[operator](lvalue, rvalue);
  return result ? options.fn(this) : options.inverse(this);
}

/**
 * separate array
 * @param  {String|Array} value     input value
 * @param  {String}       separator separate symbol
 * @return {String}
 */
function separate(value, separator) {
  if (3 > arguments.length) {
    separator = ' ';
  }

  if (_lodash2.default.isString(value)) {
    return value;
  }

  if (_lodash2.default.isArray(value)) {
    return value.join(separator);
  }
}