'use strict';

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _colors = require('colors');

var _colors2 = _interopRequireDefault(_colors);

var _commander = require('commander');

var _commander2 = _interopRequireDefault(_commander);

var _option_merger = require('./libs/option_merger');

var _option_merger2 = _interopRequireDefault(_option_merger);

var _utils = require('./libs/utils');

var utils = _interopRequireWildcard(_utils);

var _variables = require('./conf/variables');

var VARS = _interopRequireWildcard(_variables);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * node module colors will be changed by karma
 * if you import karma, attribute bold in colors
 * will be change to function and can not use
 * 'string'.bold, it must be write it to 'string'.bold()
 * so that every time after require karma, you can not be
 * use bold for string attribute, it can not return string
 * it will be return a function.
 */

var params = process.argv;
var cwd = _path2.default.basename(require.main.filename);
var pkg = _path2.default.join(__dirname, '../package.json');
var source = _fsExtra2.default.readJSONSync(pkg);

/**
 * version setting
 */
_commander2.default.version(source.version).option('--quiet');

/**
 * init command
 */
_commander2.default.command('init <name>').description('Create a new ngwp project').option('--ver <project version>', 'Set project version').option('--description <project description>', 'Set project description').action(function (name, options) {
  var _require = require('./libs/initialization'),
      initialize = _require.initialize;

  var startTime = Date.now();
  var folder = _path2.default.join(_option_merger2.default.ROOT_PATH, name);

  if (_fsExtra2.default.exists(folder)) {
    throw new Error(folder + ' is exists');
  }

  _fsExtra2.default.mkdirSync(folder);

  initialize(name, {
    dist: folder,
    version: options.ver || '0.0.1',
    description: options.description || name
  }, function (error, stats) {
    /* istanbul ignore if */
    if (error) {
      throw error;
    }

    utils.trace('Generator: installer');
    utils.trace('Time: ' + _colors2.default.bold(_colors2.default.white(Date.now() - startTime)) + 'ms\n');

    utils.printStats(stats);
  });
}).on('--help', function () {
  utils.trace('  Examples:');
  utils.trace('    $ ' + cwd + ' init myProject');
  utils.trace('');
});

/**
 * module command
 */
_commander2.default.command('module <name>').description('Create a new module (Every module is entrance of SPA)').option('-d, --dist <filename>', 'Set destination file').option('-b, --base <folder>', 'Set destination base path').action(function (name, options) {
  var _require2 = require('./libs/builder'),
      mkModule = _require2.mkModule;

  var startTime = Date.now();

  mkModule(name, {
    basePath: options.base || _option_merger2.default.ROOT_PATH,
    distFolder: options.dist || _path2.default.join(_option_merger2.default.RESOURCE_FOLDER_NAME, _option_merger2.default.ENTRY_FOLDER_NAME)
  }, function (error, stats) {
    /* istanbul ignore if */
    if (error) {
      throw error;
    }

    utils.trace('Generator: module');
    utils.trace('Time: ' + _colors2.default.white(Date.now() - startTime).bold + 'ms\n');

    utils.printStats(stats);
  });
}).on('--help', function () {
  utils.trace('  Examples:');
  utils.trace('    $ ' + cwd + ' module myModule');
  utils.trace('');
});

/**
 * route command
 */
_commander2.default.command('route <module> <routes>').description('Create components by route').option('-d, --dist <filename>', 'Set destination file').option('-b, --base <folder>', 'Set destination base path').action(function (module, routes, options) {
  var _require3 = require('./libs/builder'),
      mkRoute = _require3.mkRoute;

  var startTime = Date.now();

  mkRoute(routes, module, {
    basePath: options.base || _option_merger2.default.ROOT_PATH,
    distFolder: options.dist || _path2.default.join(_option_merger2.default.RESOURCE_FOLDER_NAME, _option_merger2.default.ENTRY_FOLDER_NAME)
  }, function (error, stats) {
    /* istanbul ignore if */
    if (error) {
      throw error;
    }

    utils.trace('Generator: route');
    utils.trace('Time: ' + _colors2.default.white(Date.now() - startTime).bold + 'ms\n');
    utils.printStats(stats);
  });
}).on('--help', function () {
  utils.trace('  Examples:');
  utils.trace('    $ ' + cwd + ' route myModule route1/route2/route3');
  utils.trace('');
});

/**
 * vhosts command
 */
_commander2.default.command('vhosts').description('Generate nginx vhosts config file with modules').option('-c, --config', 'Set module config (Default path/to/conf/config.js)').option('-d, --dist <filename>', 'Set destination file').option('-b, --base <folder>', 'Set destination base path').option('-p, --port <webpack server port>', 'Set webpack develop server port in development').option('--root-path <Root folder>', 'Set variable \'root\' in nginx conf (Default destination folder)').option('--logs-path <Logs folder>', 'Set log folder in nginx conf (Default \'base/logs/\')').option('--use-https', 'Use https protocol (Default false)').option('--cert-path', 'Set root cert path (Default base folder)').option('--cert-file', 'Set cert file (Require when --use-https is open)').option('--cert-key', 'Set cert key file (Require when --use-https is true)').action(function (options) {
  var _require4 = require('./libs/vhosts'),
      mkVhost = _require4.mkVhost;

  var startTime = Date.now();

  if (options.port) {
    options.port = options.port * 1;

    _lodash2.default.forEach(_option_merger2.default.NGINX_PROXY, function (proxy) {
      proxy.proxyPort = options.port;
    });

    _option_merger2.default.updateRC({ port: options.port });
  }

  if (true === options.useHttps) {
    var certPath = options.certPath || process.env.NGWP_CERT_PATH || process.env.ngwp_cert_path || VARS.ROOT_PATH;

    _lodash2.default.forEach(_option_merger2.default.NGINX_PROXY, function (setting) {
      if (true === setting.useHttps) {
        setting.certFile = setting.certFile ? utils.resolvePath(setting.certFile, certPath) : _path2.default.join(certPath, setting.domain + '.pem');

        setting.certKey = setting.certFile ? utils.resolvePath(setting.certFile, certPath) : _path2.default.join(certPath, setting.domain + '.key');

        if (!(_fsExtra2.default.existsSync(setting.certFile) && _fsExtra2.default.existsSync(setting.certKey))) {
          utils.trace('[' + _colors2.default.yellow('warn') + '] \'' + setting.certFile + '\' or \'' + setting.certKey + '\' is not found');
          setting.useHttps = false;
        }
      }
    });
  }

  mkVhost(_option_merger2.default.NGINX_PROXY, {
    basePath: options.base,
    distFile: options.dist,
    rootPath: options.rootPath,
    logsPath: options.logsPath
  }, function (error, stats) {
    /* istanbul ignore if */
    if (error) {
      throw error;
    }

    var file = stats.file,
        modules = stats.modules;


    modules = _lodash2.default.filter(modules, function (module) {
      return !(_lodash2.default.isEmpty(module.domain) || _lodash2.default.isEmpty(module.proxy) || _lodash2.default.isEmpty(module.entries));
    });

    modules = _lodash2.default.map(modules, function (module) {
      return _lodash2.default.pick(module, ['domain', 'proxy', 'entries']);
    });

    utils.trace('Generator: vhosts');
    utils.trace('Time: ' + _colors2.default.bold(_colors2.default.white(Date.now() - startTime)) + 'ms\n');

    if (utils.printStats(modules)) {
      utils.trace('[' + _colors2.default.bold(_colors2.default.green('ok')) + '] Nginx config file \'' + _colors2.default.bold(_colors2.default.green(file)) + '\' is generated successfully');
      utils.trace('Remember include it and \'' + _colors2.default.bold(_colors2.default.green('reolad/restart')) + '\' your nginx server');
    }
  });
}).on('--help', function () {
  utils.trace('  Examples:');
  utils.trace('    $ ' + cwd + ' vhosts');
  utils.trace('');
});

var runCommander = _commander2.default.command('run <mode>').description('Start webpack').action(function (mode) {
  if (-1 !== _lodash2.default.indexOf(['dev', 'develop', 'development'], mode)) {
    runDevelopTasks();
  } else if (-1 !== _lodash2.default.indexOf(['prod', 'product', 'production'], mode)) {
    runReleaseTasks();
  } else if (-1 !== _lodash2.default.indexOf(['test', 'unitest'], mode)) {
    runUnitestTasks();
  } else {
    runCommander.help();

    process.exit(0);
  }
}).on('--help', function () {
  utils.trace('  Examples:');
  utils.trace('    $ ' + cwd + ' run develop');
  utils.trace('    $ ' + cwd + ' run product');
  utils.trace('    $ ' + cwd + ' run unitest');
  utils.trace('');
});

/**
 * other return helper
 */
_commander2.default.action(function () {
  _commander2.default.help();
  process.exit(0);
});

!params.slice(2).length ? runDevelopTasks() : _commander2.default.parse(params);

/**
 * convert and package
 * run webpack develop server and watch the file changed
 */
function runDevelopTasks() {
  process.env.DEVELOP = 1;

  var _require5 = require('./libs/webpack'),
      run = _require5.run;

  run(_path2.default.join(__dirname, './conf/webpack.develop.config.babel.js'), { watch: true });
}

/**
 * convert and package
 * minify all we can compress
 */
function runReleaseTasks() {
  process.env.PRODUCT = 1;

  var _require6 = require('./libs/webpack'),
      run = _require6.run;

  run(_path2.default.join(__dirname, './conf/webpack.product.config.babel.js'));
}

/**
 * convert and run karma
 * import all test/*.spec.js files
 */
function runUnitestTasks() {
  process.env.PRODUCT = 1;
  process.env.UNITEST = 1;

  var _require7 = require('./libs/karma'),
      run = _require7.run;

  run(_path2.default.join(__dirname, './conf/karma.conf.babel.js'));
}